// database row conversion                                       -*- C++ -*-
// $Id: rowconvert.h,v 1.4 2004/01/07 14:29:39 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PQXX_OBJECT_ROWCONVERT_H
#define PQXX_OBJECT_ROWCONVERT_H

#include <vector>

#include <pqxx-object/transaction.h>

namespace pqxxobject
{
  /**
   * Database row conversion.
   * This class (function object) represents the operations that may
   * be performed by a single row in a database table.  This means all
   * conversions between a "row object" class and a row in a database
   * table (or result set) such as selection, insertion, update and
   * deletion.
   *
   * This class should be inherited by all row classes.
   */
  class rowconvert
  {
  public:
    /// The constructor.
    rowconvert()
    {}

    /// The destructor.
    virtual ~rowconvert()
    {}

    /// Operation to perform on the database.
    enum operation_type
      {
	/// Insert a row into the database table.
	OPERATION_INSERT,
	/// Update a row in the database table.
	OPERATION_UPDATE,
	/// Delete a row from the database table.
	OPERATION_DELETE
      };

    /**
     * Create a "row object" from a row of a result set.
     * This method converts a row of a result set into an object.
     * @param row an iterator pointing to a row of a result set.
     * @param tran the transaction which was used to return row.  This
     * is used for recursive initialisation of objects e.g. when
     * contained or derived-from objects must also be initialised.
     * @returns a pointer to a row object, or NULL if the conversion
     * failed.
     */
    virtual void initialise(pqxx::result::const_iterator row,
			    transaction& tran) = 0;

    /**
     * Insert a row into a table.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void insert(transaction& tran) = 0;

    /**
     * Update a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void update(transaction& tran) = 0;

    /**
     * Remove a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void erase(transaction& tran) = 0;

    /**
     * Refresh a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void refresh(transaction& tran) = 0;

  }; // class rowconvert

}; // namespace pqxxobject

#endif // PQXX_OBJECT_ROWCONVERT_H
